@@ -23,6 +23,18 @@
 #
 # ============================================================
 
+"""
+This package contains all classes that form the high-level user interface of the
+RSB python implementation. It is the entry point for most users and only in
+advanced cases client programs need to use classes from other modules.
+
+In order to create basic objects have a look at the functions L{createInformer},
+L{createListener}, L{createServer} and L{createRemoteServer}.
+
+@author: jwienke
+@author: jmoringe
+"""
+
 import uuid
 import copy
 import logging
@@ -45,15 +57,15 @@ except ImportError:
     pass
 
 def haveSpread():
-    '''
+    """
     Indicates whether the installation of RSB has spread support.
-    
+
     @return: True if spread is available, else False
-    '''
+    """
     return _spreadAvailable
 
 class QualityOfServiceSpec(object):
-    '''
+    """
     Specification of desired quality of service settings for sending
     and receiving events. Specification given here are required "at
     least". This means concrete connector implementations can provide
@@ -62,57 +74,57 @@ class QualityOfServiceSpec(object):
     values mean better services.
 
     @author: jwienke
-    '''
+    """
 
     Ordering = Enum("Ordering", ["UNORDERED", "ORDERED"], [10, 20])
     Reliability = Enum("Reliability", ["UNRELIABLE", "RELIABLE"], [10, 20])
 
     def __init__(self, ordering=Ordering.UNORDERED, reliability=Reliability.RELIABLE):
-        '''
+        """
         Constructs a new QoS specification with desired
         details. Defaults are unordered but reliable.
 
         @param ordering: desired ordering type
         @param reliability: desired reliability type
-        '''
+        """
         self.__ordering = ordering
         self.__reliability = reliability
 
     def getOrdering(self):
-        '''
+        """
         Returns the desired ordering settings.
 
         @return: ordering settings
-        '''
+        """
 
         return self.__ordering
 
     def setOrdering(self, ordering):
-        '''
+        """
         Sets the desired ordering settings
 
         @param ordering: ordering to set
-        '''
+        """
 
         self.__ordering = ordering
 
     ordering = property(getOrdering, setOrdering)
 
     def getReliability(self):
-        '''
+        """
         Returns the desired reliability settings.
 
         @return: reliability settings
-        '''
+        """
 
         return self.__reliability
 
     def setReliability(self, reliability):
-        '''
+        """
         Sets the desired reliability settings
 
         @param reliability: reliability to set
-        '''
+        """
 
         self.__reliability = reliability
 
@@ -131,29 +143,29 @@ class QualityOfServiceSpec(object):
         return "%s(%r, %r)" % (self.__class__.__name__, self.__ordering, self.__reliability)
 
 class ParticipantConfig (object):
-    '''
+    """
     Objects of this class describe desired configurations for newly
     created L{Participant}s with respect to:
-    - Quality of service settings
-    - Error handling strategies (not currently used)
-    - Employed transport mechanisms
-      - Their configurations (e.g. port numbers)
-      - Associated converters
+     - Quality of service settings
+     - Error handling strategies (not currently used)
+     - Employed transport mechanisms
+       - Their configurations (e.g. port numbers)
+       - Associated converters
 
     @author: jmoringe
-    '''
+    """
 
     class Transport (object):
-        '''
+        """
         Objects of this class describe configurations of transports
         connectors. These consist of
-        - Transport name
-        - Enabled vs. Disabled
-        - Optional converter selection
-        - Transport-specific options
+         - Transport name
+         - Enabled vs. Disabled
+         - Optional converter selection
+         - Transport-specific options
 
         @author: jmoringe
-        '''
+        """
         def __init__(self, name, options={}, converter_ = None):
             import rsb.converter
             self.__name = name
@@ -198,7 +210,7 @@ class ParticipantConfig (object):
 
         def isEnabled(self):
             return self.__enabled
-        
+
         def setEnabled(self, flag):
             self.__enabled = flag
 
@@ -290,7 +302,7 @@ class ParticipantConfig (object):
 
     @classmethod
     def fromFile(clazz, path, defaults={}):
-        '''
+        """
         Obtain configuration options from the configuration file
         B{path}, store them in a L{ParticipantConfig} object and
         return it.
@@ -309,7 +321,7 @@ class ParticipantConfig (object):
         @rtype: ParticipantConfig
 
         See also: L{fromEnvironment}, L{fromDefaultSources}
-        '''
+        """
         return clazz.__fromDict(clazz.__fromFile(path, defaults))
 
     @classmethod
@@ -322,7 +334,7 @@ class ParticipantConfig (object):
 
     @classmethod
     def fromEnvironment(clazz, defaults={}):
-        '''
+        """
         Obtain configuration options from environment variables, store
         them in a L{ParticipantConfig} object and return
         it. Environment variable names are mapped to RSB option names
@@ -340,20 +352,20 @@ class ParticipantConfig (object):
         @rtype: ParticipantConfig
 
         See also: L{fromFile}, L{fromDefaultSources}
-        '''
+        """
         return clazz.__fromDict(clazz.__fromEnvironment(defaults))
 
     @classmethod
     def fromDefaultSources(clazz, defaults={}):
-        '''
+        """
         Obtain configuration options from multiple sources, store them
         in a L{ParticipantConfig} object and return it. The following
         sources of configuration information will be consulted:
 
-        1. C{/etc/rsb.conf}
-        2. C{~/.config/rsb.conf}
-        3. C{\$(PWD)/rsb.conf}
-        4. Environment Variables
+         1. C{/etc/rsb.conf}
+         2. C{~/.config/rsb.conf}
+         3. C{\$(PWD)/rsb.conf}
+         4. Environment Variables
 
         @param defaults: A L{ParticipantConfig} object the options of
                          which should be used as defaults.
@@ -364,7 +376,7 @@ class ParticipantConfig (object):
         @rtype: ParticipantConfig
 
         See also: L{fromFile}, L{fromEnvironment}
-        '''
+        """
         defaults = {"transport.socket.enabled": "1"}
         if platform.system() == 'Windows':
             partial = clazz.__fromFile("c:\\rsb.conf", defaults)
@@ -376,12 +388,12 @@ class ParticipantConfig (object):
         return clazz.__fromDict(options)
 
 class Scope(object):
-    '''
+    """
     A scope defines a channel of the hierarchical unified bus covered by RSB.
     It is defined by a surface syntax like C{"/a/deep/scope"}.
 
     @author: jwienke
-    '''
+    """
 
     __COMPONENT_SEPARATOR = "/"
     __COMPONENT_REGEX = re.compile("^[a-zA-Z0-9]+$")
@@ -394,14 +406,14 @@ class Scope(object):
             return Scope(thing)
 
     def __init__(self, stringRep):
-        '''
+        """
         Parses a scope from a string representation.
 
         @param stringRep: string representation of the scope
         @type stringRep: str or unicode
         @raise ValueError: if B{stringRep} does not have the right
                            syntax
-        '''
+        """
 
         if len(stringRep) == 0:
             raise ValueError("The empty string does not designate a scope; Use '/' to designate the root scope.")
@@ -432,7 +444,7 @@ class Scope(object):
                 raise ValueError("Invalid character in component %s. Given was scope '%s'." % (com, stringRep))
 
     def getComponents(self):
-        '''
+        """
         Returns all components of the scope as an ordered list. Components are
         the names between the separator character '/'. The first entry in the
         list is the highest level of hierarchy. The scope '/' returns an empty
@@ -441,19 +453,19 @@ class Scope(object):
         @return: components of the represented scope as ordered list with highest
                  level as first entry
         @rtype: list
-        '''
+        """
         return copy.copy(self.__components)
 
     components = property(getComponents)
 
     def toString(self):
-        '''
+        """
         Reconstructs a fully formal string representation of the scope with
         leading an trailing slashes.
 
         @return: string representation of the scope
         @rtype: str
-        '''
+        """
 
         string = self.__COMPONENT_SEPARATOR
         for com in self.__components:
@@ -462,7 +474,7 @@ class Scope(object):
         return string
 
     def concat(self, childScope):
-        '''
+        """
         Creates a new scope that is a sub-scope of this one with the
         subordinated scope described by the given
         argument. E.g. C{"/this/is/".concat("/a/test/")} results in
@@ -473,14 +485,14 @@ class Scope(object):
         @type childScope: Scope
         @return: new scope instance representing the created sub-scope
         @rtype: Scope
-        '''
+        """
         newScope = Scope("/")
         newScope.__components = copy.copy(self.__components)
         newScope.__components += childScope.__components
         return newScope
 
     def isSubScopeOf(self, other):
-        '''
+        """
         Tests whether this scope is a sub-scope of the given other scope, which
         means that the other scope is a prefix of this scope. E.g. "/a/b/" is a
         sub-scope of "/a/".
@@ -490,7 +502,7 @@ class Scope(object):
         @return: C{True} if this is a sub-scope of the other scope, equality gives
                  C{False}, too
         @rtype: Bool
-        '''
+        """
 
         if len(self.__components) <= len(other.__components):
             return False
@@ -498,7 +510,7 @@ class Scope(object):
         return other.__components == self.__components[:len(other.__components)]
 
     def isSuperScopeOf(self, other):
-        '''
+        """
         Inverse operation of L{isSubScopeOf}.
 
         @param other: other scope to test
@@ -506,7 +518,7 @@ class Scope(object):
         @return: C{True} if this scope is a strict super scope of the other scope.
                  equality also gives C{False}.
         @rtype: Bool
-        '''
+        """
 
         if len(self.__components) >= len(other.__components):
             return False
@@ -514,7 +526,7 @@ class Scope(object):
         return self.__components == other.__components[:len(self.__components)]
 
     def superScopes(self, includeSelf=False):
-        '''
+        """
         Generates all super scopes of this scope including the root
         scope "/".  The returned list of scopes is ordered by
         hierarchy with "/" being the first entry.
@@ -527,7 +539,7 @@ class Scope(object):
         @return: list of all super scopes ordered by hierarchy, "/"
                  being first
         @rtype: list of Scopes
-        '''
+        """
 
         supers = []
 
@@ -535,9 +547,9 @@ class Scope(object):
         if not includeSelf:
             maxIndex -= 1
         for i in range(maxIndex + 1):
-            super = Scope("/")
-            super.__components = self.__components[:i]
-            supers.append(super)
+            superScope = Scope("/")
+            superScope.__components = self.__components[:i]
+            supers.append(superScope)
 
         return supers
 
@@ -590,10 +602,11 @@ class MetaData (object):
                             the associated event was delivered to the
                             user-level handler by RSB.
         @param userTimes: A dictionary of user-supplied timestamps.
-        @type userTime: dict
+        @type userTimes: dict from string name to double value as seconds since
+                         unix epoche
         @param userInfos: A dictionary of user-supplied meta-data
                           items.
-        @type userInfos: dict
+        @type userInfos: dict from string to string
         """
         if createTime is None:
             self.__createTime = time.time()
@@ -699,12 +712,12 @@ class MetaData (object):
         return self.__str__()
 
 class EventId(object):
-    '''
+    """
     Uniquely identifies an Event by the sending participants ID and a sequence
     number within this participant. Optional conversion to uuid is possible.
 
     @author: jwienke
-    '''
+    """
 
     def __init__(self, participantId, sequenceNumber):
         self.__participantId = participantId
@@ -784,7 +797,7 @@ class EventId(object):
         return result;
 
 class Event(object):
-    '''
+    """
     Basic event class.
 
     Events are often caused by other events, which e.g. means that their
@@ -798,7 +811,7 @@ class Event(object):
     of Events, Addison-Wessley, 2007
 
     @author: jwienke
-    '''
+    """
 
     def __init__(self, id = None, scope = Scope("/"), method = None,
                  data = None, type = None,
@@ -816,15 +829,17 @@ class Event(object):
                        communication patters. Examples are
                        C{"REQUEST"} and C{"REPLY"}.
         @type method: str
-        @param data:
-        @param type:
+        @param data: data contained in this event
+        @param type: python data type of the contained data
         @type type: type
-        @param metaData:
+        @param metaData: meta data to use for the new event
         @type metaData: MetaData
-        @param userInfos:
-        @type userInfos: dict
-        @param userTimes:
-        @type userTime: dict
+        @param userInfos: key-value like store of user infos to add to the meta
+                          data of this event
+        @type userInfos: dict from string to string
+        @param userTimes: additional timestamps to add to the meta data
+        @type userTimes: dict from string timestamp name to value of timestamp
+                         as dobule of seconds unix epoch
         @param causes: A list of L{EventId}s of events which causes the
                        newly constructed events.
         @type causes: list
@@ -875,8 +890,8 @@ class Event(object):
             raise RuntimeError("The event does not have an ID so far.")
         return self.__id
 
-    def setId(self, id):
-        self.__id = id
+    def setId(self, theId):
+        self.__id = theId
 
     id = property(getId, setId)
 
@@ -962,14 +977,14 @@ class Event(object):
 
         return self.__type
 
-    def setType(self, type):
+    def setType(self, theType):
         """
         Sets the type of the user data of this event
 
-        @param type: user data type
+        @param theType: user data type
         """
 
-        self.__type = type
+        self.__type = theType
 
     type = property(getType, setType)
 
@@ -981,64 +996,64 @@ class Event(object):
 
     metaData = property(getMetaData, setMetaData)
 
-    def addCause(self, id):
-        '''
+    def addCause(self, theId):
+        """
         Adds a causing EventId to the causes of this event.
 
-        @param id: id to add
-        @type id: EventId
+        @param theId: id to add
+        @type theId: EventId
         @return: true if the id was newly added, else false
         @rtype: bool
-        '''
-        if id in self.__causes:
+        """
+        if theId in self.__causes:
             return False
         else:
-            self.__causes.append(id)
+            self.__causes.append(theId)
             return True
 
-    def removeCause(self, id):
-        '''
+    def removeCause(self, theId):
+        """
         Removes a causing EventId from the causes of this event.
 
-        @param id: id to remove
-        @type id: EventId
+        @param theId: id to remove
+        @type theId: EventId
         @return: true if the id was remove, else false (because it did not exist)
         @rtype: bool
-        '''
-        if id in self.__causes:
-            self.__causes.remove(id)
+        """
+        if theId in self.__causes:
+            self.__causes.remove(theId)
             return True
         else:
             return False
 
-    def isCause(self, id):
-        '''
+    def isCause(self, theId):
+        """
         Checks whether a given id of an event is marked as a cause for this
         event.
 
-        @param id: id to check
-        @type id: EventId
+        @param theId: id to check
+        @type theId: EventId
         @return: true if the id is a cause of this event, else false
         @rtype: bool
-        '''
-        return id in self.__causes
+        """
+        return theId in self.__causes
 
     def getCauses(self):
-        '''
+        """
         Returns all causes of this event.
 
         @return: causing event ids
         @rtype: list of EventIds
-        '''
+        """
         return self.__causes
 
     def setCauses(self, causes):
-        '''
+        """
         Overwrites the cause vector of this event with the given one.
 
         @param causes: new cause vector
         @type causes: list of EventId
-        '''
+        """
         self.__causes = causes
 
     causes = property(getCauses, setCauses)
@@ -1086,8 +1101,8 @@ class Participant(object):
     def getId(self):
         return self.__id
 
-    def setId(self, id):
-        self.__id = id
+    def setId(self, theId):
+        self.__id = theId
 
     id = property(getId, setId)
 
@@ -1100,47 +1115,46 @@ class Participant(object):
     scope = property(getScope, setScope)
 
     @classmethod
-    def getConnector(clazz, direction, config):
+    def getConnectors(clazz, direction, config):
         if not direction in ('in', 'out'):
             raise ValueError, 'Invalid direction: %s (valid directions are "in" and "out")' % direction
         if len(config.getTransports()) == 0:
             raise ValueError, 'No transports specified (config is %s)' \
                 % config
-        if len(config.getTransports()) > 1:
-            raise ValueError, 'Multiple transports are not supported, yet (config has %d transports, config is %s)' \
-                % (len(config.getTransports()), config)
 
-        transport = config.getTransports()[0]
-        if transport.getName() == 'spread':
-            if not haveSpread():
-                raise ValueError, "Spread transport not enabled as the python spread module cannot be found in the running interpreter"
-            from transport import rsbspread
-            if direction == 'in':
-                klass = rsbspread.InConnector
-            elif direction == 'out':
-                klass = rsbspread.OutConnector
+        transports = []
+        for transport in config.getTransports():
+            if transport.getName() == 'spread':
+                if not haveSpread():
+                    raise ValueError, "Spread transport not enabled as the python spread module cannot be found in the running interpreter"
+                from transport import rsbspread
+                if direction == 'in':
+                    klass = rsbspread.InConnector
+                elif direction == 'out':
+                    klass = rsbspread.OutConnector
+                else:
+                    assert(False)
+            elif transport.getName() == 'socket':
+                import rsb.transport.socket
+                if direction == 'in':
+                    klass = rsb.transport.socket.InPushConnector
+                elif direction == 'out':
+                    klass = rsb.transport.socket.OutConnector
+                else:
+                    assert(False)
+            elif transport.getName() == 'inprocess':
+                import rsb.transport.local
+                if direction == 'in':
+                    klass = rsb.transport.local.InConnector
+                elif direction == 'out':
+                    klass = rsb.transport.local.OutConnector
+                else:
+                    assert(False)
             else:
-                assert(False)
-        elif transport.getName() == 'socket':
-            import rsb.transport.socket
-            if direction == 'in':
-                klass = rsb.transport.socket.InConnector
-            elif direction == 'out':
-                klass = rsb.transport.socket.OutConnector
-            else:
-                assert(False)
-        elif transport.getName() == 'inprocess':
-            import rsb.transport.local
-            if direction == 'in':
-                klass = rsb.transport.local.InConnector
-            elif direction == 'out':
-                klass = rsb.transport.local.OutConnector
-            else:
-                assert(False)
-        else:
-            raise ValueError, 'No such transport: "%s"' % transport.getName()
-        return klass(converters = transport.getConverters(),
-                     options    = transport.getOptions())
+                raise ValueError, 'No such transport: "%s"' % transport.getName()
+            transports.append(klass(converters = transport.getConverters(),
+                                    options    = transport.getOptions()))
+        return transports
 
 class Informer(Participant):
     """
@@ -1149,7 +1163,7 @@ class Informer(Participant):
     @author: jwienke
     """
 
-    def __init__(self, scope, type,
+    def __init__(self, scope, theType,
                  config       = None,
                  configurator = None):
         """
@@ -1158,10 +1172,10 @@ class Informer(Participant):
 
         @param scope: scope of the informer
         @type scope: Scope
-        @param type: A Python object designating the type of objects
-                     that will be sent via the new informer. Instances
-                     of subtypes are permitted as well.
-        @type type: type
+        @param theType: A Python object designating the type of objects
+                        that will be sent via the new informer. Instances
+                        of subtypes are permitted as well.
+        @type theType: type
         @param configurator: Out route configurator to manage sending
                              of events through out connectors.
         @todo: maybe provide an automatic type identifier deduction for default
@@ -1171,29 +1185,32 @@ class Informer(Participant):
 
         self.__logger = getLoggerByClass(self.__class__)
 
+        # TODO check that type can be converted
+        self.__type           = theType
+        self.__sequenceNumber = 0
+        self.__configurator   = None
+        
+        self.__active         = False
+        self.__mutex          = threading.Lock()
+
         if config is None:
             config = getDefaultParticipantConfig()
 
         if configurator:
             self.__configurator = configurator
         else:
-            connector = self.getConnector('out', config)
-            connector.setQualityOfServiceSpec(config.getQualityOfServiceSpec())
-            self.__configurator = rsb.eventprocessing.OutRouteConfigurator(connectors = [ connector ])
+            connectors = self.getConnectors('out', config)
+            for connector in connectors:
+                connector.setQualityOfServiceSpec(config.getQualityOfServiceSpec())
+            self.__configurator = rsb.eventprocessing.OutRouteConfigurator(connectors = connectors)
         self.__configurator.setQualityOfServiceSpec(config.getQualityOfServiceSpec())
-        # TODO check that type can be converted
-        self.__type = type
-
-        self.__sequenceNumber = 0
-
-        self.__active = False
-        self.__mutex = threading.Lock()
 
         self.__activate()
 
     def __del__(self):
         self.__logger.debug("Destructing Informer")
-        self.deactivate()
+        if self.__active:
+            self.deactivate()
 
     def getType(self):
         """
@@ -1282,27 +1299,30 @@ class Listener(Participant):
 
         self.__logger = getLoggerByClass(self.__class__)
 
+        self.__filters      = []
+        self.__handlers     = []
+        self.__configurator = None
+        self.__active       = False
+        self.__mutex        = threading.Lock()
+
         if config is None:
             config = getDefaultParticipantConfig()
 
         if configurator:
             self.__configurator = configurator
         else:
-            connector = self.getConnector('in', config)
-            connector.setQualityOfServiceSpec(config.getQualityOfServiceSpec())
-            self.__configurator = rsb.eventprocessing.InRouteConfigurator(connectors = [ connector ])
+            connectors = self.getConnectors('in', config)
+            for connector in connectors:
+                connector.setQualityOfServiceSpec(config.getQualityOfServiceSpec())
+            self.__configurator = rsb.eventprocessing.InRouteConfigurator(connectors = connectors)
         self.__configurator.setScope(scope)
 
-        self.__mutex = threading.Lock()
-        self.__active = False
-
-        self.__filters = []
-        self.__handlers = []
 
         self.__activate()
 
     def __del__(self):
-        self.deactivate()
+        if self.__active:
+            self.deactivate()
 
     def __activate(self):
         # TODO commonality with Informer... refactor
@@ -1323,7 +1343,7 @@ class Listener(Participant):
             else:
                 self.__logger.info("Deactivate called even though listener was not active")
 
-    def addFilter(self, filter):
+    def addFilter(self, theFilter):
         """
         Appends a filter to restrict the events received by this listener.
 
@@ -1331,8 +1351,8 @@ class Listener(Participant):
         """
 
         with self.__mutex:
-            self.__filters.append(filter)
-            self.__configurator.filterAdded(filter)
+            self.__filters.append(theFilter)
+            self.__configurator.filterAdded(theFilter)
 
     def getFilters(self):
         """
